#!/usr/bin/perl -w

=head1 NAME

B<Regexp::Common::RealHTML> - REGEX    HTML

=head1 SYNOPSIS

 use Regexp::Common qw/ RealHTML /;
 use Regexp::Common::Fast;	    #  

=head1 DESCRIPTION

  "REAL HTML"   "HTML 4.0":
 -        '', "", ``   .
 -        [^'"`\s>=].
 -        [^'"`\s>].
 -          
     =.
 -     : <TAG =value>.
 -  :
   *  ``<!--''   ``-->'',  ``--''  
   *  : ``<!-->''  ``<!--->''
   *  ``-->''  ,   ``<!--''   ``>''
   *  ``<?''   ``?>''
   *  : ``<?>''
 -    ``/''   ``>''

=head1 REGEX ATOMS

     : Regexp::Common (B<$RE{something}>)
 Regexp::Common::Fast (B<\R[something]>).

=over 4

=cut

package Regexp::Common::RealHTML;

#use strict;
#no strict 'subs'; # Bareword "Apache" not allowed: "$RE{x}{-word=>Apache}"

our $VERSION = '1.05';

use Regexp::Common qw /pattern clean no_defaults/;

#    :  ``TYPE''  ``"TYPE"''
my $pname   = qr/ (?:  (?<=\s)  [\"\'\`]?  (
		    (?<=\s)  [^\"\'\`\s>=]+  (?=[\s>=])	|
		    (?<=\")  [^\"]+	     (?=\")	|
		    (?<=\')  [^\']+	     (?=\')	|
		    (?<=\`)  [^\`]+	     (?=\`)
		    )  [\"\'\`]?  ) /x;

#    : ``VALUE''  ``"VALUE"''
my $pval    = qr/ (?: (?<=\=)  [\"\'\`]?  (
		    (?<=\=)  [^\"\'\`\s>]*   (?=[\s>])	|
		    (?<=\")  [^\"]*	     (?=\")	|
		    (?<=\')  [^\']*	     (?=\')	|
		    (?<=\`)  [^\`]*	     (?=\`)
		    )  [\"\'\`]? ) /x;

#   : `` =''  `` TYPE=''  `` =VALUE''  `` TYPE=VALUE''
#  `` TYPE''
my $param   = qr/ (?: \s+ (?: $pname(?:=$pval?)? | =$pval? ) ) /xo;

=item B<$RE{pname} \R[pname]>

    HTML-.
 {-keep}      (  )  $1.

=item B<$RE{pval} \R[pval]>

    HTML-.
 {-keep}      (  )  $1.

=item B<$RE{param} \R[param]>

 HTML-.

=cut

#      qr//: ``Sequence (?k...) not recognized in regex''
s/\((?=\s)/(?k:/g for $pname, $pval, $param;
pattern name => [ 'pname'   ], create => $pname;
pattern name => [ 'pval'    ], create => $pval;
pattern name => [ 'param'   ], create => $param;

#    <>
my $tag	    = qr/ (?: [!\/]? \w+ $RE{param}* \s* ) /xo;

#      <>: ``TITLE>....</TITLE''
my $ctag    = qr# (?:
    TITLE  $RE{param}* \s* /? > [^<]* (?:<(?!/TITLE) [^<]*)* </TITLE  $RE{param}* \s* |
    STYLE  $RE{param}* \s* /? > [^<]* (?:<(?!/STYLE) [^<]*)* </STYLE  $RE{param}* \s* |
    SCRIPT $RE{param}* \s* /? > [^<]* (?:<(?!/SCRIPT)[^<]*)* </SCRIPT $RE{param}* \s* |
    APPLET $RE{param}* \s* /? > [^<]* (?:<(?!/APPLET)[^<]*)* </APPLET $RE{param}* \s* |
    OBJECT $RE{param}* \s* /? > [^<]* (?:<(?!/OBJECT)[^<]*)* </OBJECT $RE{param}* \s*
    ) #xoi;

#    <>
my $comment = qr/ (?:
    !-- [^>]* (?: (?:(?<!--)>[^>]*)* (?<=--) )?  |
    \?  [^>]*     (?:(?<!\?)>[^>]*)* (?<=\?)
    ) /x;

=item B<$RE{tag} \R[tag]>

   E<lt>  E<gt>.

=item B<$RE{ctag} \R[ctag]>

 -    E<lt>  E<gt>.
-: TITLE, STYLE, SCRIPT, APPLET, OBJECT.
:

 TITLE>Some Title</TITLE

=item B<$RE{comment} \R[comment]>

HTML-  E<lt>  E<gt>.

=cut

#      qr//: ``Sequence (?k...) not recognized in regex''
s/\((?=\s)/(?k:/g for $tag, $ctag, $comment;
pattern name => [ 'tag'	    ], create => $tag;
pattern name => [ 'ctag'    ], create => $ctag;
pattern name => [ 'comment' ], create => $comment;

#    <>
my $anytag  = qr/ (?: $RE{ctag} | $RE{tag} | $RE{comment} ) /xo;

# (!)    &nbsp;
my $nbsp = qr/ (?: &nbsp (?:;|(?!;)) ) /x;
my $s = qr/ (?: (?:\s|$nbsp)* (?!\s|$nbsp) ) /xo;

=item B<$RE{anytag} \R[anytag]>

   E<lt>  E<gt>, .. ctag  tag  comment.

=item B<$RE{s} \R[s]>

    &nbsp;   /&nbsp;.

=cut

#      qr//: ``Sequence (?k...) not recognized in regex''
s/\((?=\s)/(?k:/g for $anytag, $s;
pattern name => [ 'anytag'  ], create => $anytag;
pattern name => [ 's'	    ], create => $s;

=back

=head1 REGEX

=over 4

=item B<$RE{'E<gt>'} \R[E<gt>]>

    (B<\R[param]*>),  B<E<gt>> 
    &nbsp;  (B<\R[s]>). :

 "<IMG SRC='...'>&nbsp;" =~ /<IMG\R[>]/

=cut

pattern name	=> [ '>' ],
	create	=> $RE{param}.'*\s*/?>'.$RE{s};

=item B<$RE{'E<lt>E<gt>'} \R[E<lt>E<gt>]>

   ( B<E<lt>>  B<E<gt>>).

  -not     
 ,     .   
      (B<E<lt>>)  
case-insensitive. :

 \R[<>]
 \R[<>,-not=>tr]
 \R[<>,-not=>'tr|/table']

   "<TD></TD></TR>":
 "<TR><TD></TD></TR></TABLE>" =~ /\R[<>,-not=>'tr|/table']+/;

=cut

pattern name	=> [ '<>' ],
	create	=> sub {
            my $not = $_[1]{'-not'} ? '(?i:(?!'.$_[1]{'-not'}.'))' : '';
            '(?:<'.$not.$RE{anytag}.'/?>'.$RE{s}.')';
	};

=item B<$RE{a} \R[a]>

  B<A>    HREF    .
 $1    HREF ( {-keep}  ).

=cut

pattern name	=> [ 'a' ],
	create	=> '(?:<(?i:A)'.$RE{param}.'*\s+(?i:HREF)='.$RE{pval}{-keep}.
		   $RE{param}.'*\s*/?>'.$RE{s}.')';

=item B<$RE{NB} \R[NB]>

  (..      &nbsp; )
  (..     B<E<lt>>).
 $1         &nbsp;
( {-keep}  ).

=cut

pattern name	=> [ 'NB' ],
	create	=> '(?:'.$RE{s}.'([^<]+?)'.$RE{s}.'(?=<))';

=back

=head1 BUGS

    , ..   
  (?()yes|no), (?{ code })  (??{ code })  
        $1. :

 <TAG PARAM1="VALUE1"PARAM2="VALUE2">

=head1 HISTORY

 1.05		  16 04:27:52 EET 2005
	-   &nbsp  ;
 1.04		   8 16:33:29 EET 2003
	-   ``/''  ``>''
 1.03		  15 15:25:36 EEST 2003
	- no comments

=head1 AUTHOR

Alex Efros <powerman@asdfGroup.com>

=cut

1;
